@@ -12,7 +12,7 @@
 #include "gexiv2-metadata-private.h"
 #include <string>
 #include <glib-object.h>
-#include <exiv2/xmp.hpp>
+#include <exiv2/exiv2.hpp>
 
 G_BEGIN_DECLS
 
@@ -284,11 +284,11 @@ gchar** gexiv2_metadata_get_xmp_tag_multiple (GExiv2Metadata *self, const gchar*
             it++;
         
         if (it != xmp_data.end()) {
-            int size = it->count ();
+            auto size = it->count ();
             gchar **array = g_new (gchar*, size + 1);
             array[size] = NULL;
             
-            for (int i = 0; i < it->count (); i++)
+            for (decltype(size) i = 0; i < size; i++)
                 array[i] = g_strdup (it->toString (i).c_str ());
             
             return array;
@@ -405,7 +405,7 @@ gboolean gexiv2_metadata_register_xmp_namespace (const gchar* name, const gchar*
 
     try {
         Exiv2::XmpProperties::ns(prefix);
-    } catch (Exiv2::Error& error) {
+    } catch (Exiv2::AnyError& error) {
         // No namespace, OK to register
         Exiv2::XmpProperties::registerNs(name, prefix);
         return TRUE;
