@@ -41,6 +41,13 @@ public:
         , _error{nullptr}
         , _eof{false}
         {}
+#if EXIV2_TEST_VERSION(0,27,99)
+    using size_type = size_t;
+#else
+    using size_type = long;
+#endif
+
+    size_type _size;
 
     ~GioIo() { g_clear_object (&_is); g_clear_error (&_error); _seekable = NULL;}
 #if defined(_MSC_VER)
@@ -56,6 +63,17 @@ public:
 #endif
 
     int open() {
+        if (_seekable == nullptr)
+            return 0;
+
+        if (_size >= 0)
+            return 0;
+
+        auto position = tell();
+        seek (0, Exiv2::BasicIo::end);
+        _size = tell();
+        seek (position, Exiv2::BasicIo::beg);
+
         return 0;
     }
 
@@ -64,15 +82,18 @@ public:
     }
 
     // Writing is not supported
-    long write(const Exiv2::byte *data, long wcount) { return 0; }
-    long write(BasicIo &src) { return 0; }
+    size_type write(const Exiv2::byte *data, size_type wcount) { return 0; }
+    size_type write(BasicIo &src) { return 0; }
     int putb(Exiv2::byte data) { return EOF; }
 
-
-    Exiv2::DataBuf read(long rcount) {
+#if EXIV2_TEST_VERSION(0,27,99)
+    Exiv2::DataBuf read(size_t rcount) noexcept {
+#else
+    Exiv2::DataBuf read(long rcount) override {
+#endif
         Exiv2::DataBuf b{rcount};
 
-        long bytes_read = this->read(b.pData_, rcount);
+        auto bytes_read = this->read(b.pData_, rcount);
         if (bytes_read > 0 && bytes_read != rcount) {
             b.reset({b.pData_, bytes_read});
         }
@@ -80,7 +101,7 @@ public:
         return b;
     }
 
-    long read(Exiv2::byte *buf, long rcount) {
+    size_type read(Exiv2::byte *buf, size_type rcount) {
         GError *error = NULL;
         gssize result = 0;
 
@@ -182,7 +203,7 @@ public:
     }
 
     size_t size() const {
-        return -1;
+        return static_cast<size_t>(_size);
     }
 
     bool isopen() const {
@@ -201,6 +222,15 @@ public:
         return "GIO Wrapper";
     }
 
+#ifdef EXV_UNICODE_PATH
+    std::wstring wpath() const {
+        std::string p = path();
+        std::wstring w(p.length(), L' ');
+        std::copy(p.begin(), p.end(), w.begin());
+        return w;
+    }
+#endif
+
 #if EXIV2_TEST_VERSION(0,27,99)
     Exiv2::BasicIo::UniquePtr temporary() const {
         return Exiv2::BasicIo::UniquePtr(nullptr);
