@@ -63,11 +63,11 @@ StreamIo::ptr_type StreamIo::temporary () const {
     return ptr_type{new Exiv2::MemIo ()};
 }
 
-long StreamIo::write (const Exiv2::byte* data, long write_count) {
+StreamIo::size_type StreamIo::write (const Exiv2::byte* data, StreamIo::size_type write_count) {
     if ( ! can_write)
         return 0;
     
-    long total_written_bytes = 0;
+    StreamIo::size_type total_written_bytes = 0;
     
     while (write_count > total_written_bytes) {
     
@@ -83,7 +83,7 @@ long StreamIo::write (const Exiv2::byte* data, long write_count) {
     return total_written_bytes;
 }
 
-long StreamIo::write (Exiv2::BasicIo& src) {
+StreamIo::size_type StreamIo::write (Exiv2::BasicIo& src) {
     if ( ! can_write)
         return 0;
     
@@ -163,6 +163,18 @@ int StreamIo::close () {
     return 0;
 }
 
+#if EXIV2_TEST_VERSION(0,27,99)
+
+Exiv2::DataBuf StreamIo::read (size_t read_count) noexcept {
+    Exiv2::DataBuf buffer (read_count);
+
+    long read_bytes = read (buffer.pData_, buffer.size_);
+
+    buffer.size_ = read_bytes;
+
+    return buffer;
+}
+#else
 Exiv2::DataBuf StreamIo::read (long read_count) {
     Exiv2::DataBuf buffer (read_count);
     
@@ -172,9 +184,10 @@ Exiv2::DataBuf StreamIo::read (long read_count) {
     
     return buffer;
 }
+#endif
 
-long StreamIo::read (Exiv2::byte* buf, long read_count) {
-    long total_read_bytes = 0;
+StreamIo::size_type StreamIo::read (Exiv2::byte* buf, StreamIo::size_type read_count) {
+    StreamIo::size_type total_read_bytes = 0;
 
     while (read_count > total_read_bytes) {
         /* because of a marshalling problem on managed side, we shift the
