@@ -7,13 +7,15 @@
  * This is free software. See COPYING for details.
  */
 
-#ifndef __GEXIV2_H__
-#define __GEXIV2_H__
+#ifndef GEXIV2_H
+#define GEXIV2_H
 
 #include <gexiv2/gexiv2-metadata.h>
 #include <gexiv2/gexiv2-managed-stream.h>
 #include <gexiv2/gexiv2-preview-properties.h>
 #include <gexiv2/gexiv2-preview-image.h>
 #include <gexiv2/gexiv2-log.h>
+#include <gexiv2/gexiv2-startup.h>
+#include <gexiv2/gexiv2-version.h>
 
-#endif /* __GEXIV2_H__ */
+#endif /* GEXIV2_H */
